"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from flexprice_py import errors, models, utils
from flexprice_py._hooks import HookContext
from flexprice_py.types import OptionalNullable, UNSET
from flexprice_py.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Dict, List, Mapping, Optional, Union


class Wallets(BaseSDK):
    def get_customer_wallets(
        self,
        *,
        minus: Optional[int] = None,
        expand: Optional[str] = None,
        from_cache: Optional[bool] = False,
        id: Optional[str] = None,
        include_real_time_balance: Optional[bool] = False,
        lookup_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.components.DtoWalletResponse]:
        r"""Get Customer Wallets

        Use when resolving wallets by external customer id or lookup key (e.g. from your app's user id). Supports optional real-time balance and expand.

        :param minus: populated from x-max-live header, not query param
        :param expand:
        :param from_cache:
        :param id:
        :param include_real_time_balance:
        :param lookup_key:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.GetCustomerWalletsRequest(
            minus=minus,
            expand=expand,
            from_cache=from_cache,
            id=id,
            include_real_time_balance=include_real_time_balance,
            lookup_key=lookup_key,
        )

        req = self._build_request(
            method="GET",
            path="/customers/wallets",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getCustomerWallets",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                List[models.components.DtoWalletResponse], http_res
            )
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def get_customer_wallets_async(
        self,
        *,
        minus: Optional[int] = None,
        expand: Optional[str] = None,
        from_cache: Optional[bool] = False,
        id: Optional[str] = None,
        include_real_time_balance: Optional[bool] = False,
        lookup_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.components.DtoWalletResponse]:
        r"""Get Customer Wallets

        Use when resolving wallets by external customer id or lookup key (e.g. from your app's user id). Supports optional real-time balance and expand.

        :param minus: populated from x-max-live header, not query param
        :param expand:
        :param from_cache:
        :param id:
        :param include_real_time_balance:
        :param lookup_key:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.GetCustomerWalletsRequest(
            minus=minus,
            expand=expand,
            from_cache=from_cache,
            id=id,
            include_real_time_balance=include_real_time_balance,
            lookup_key=lookup_key,
        )

        req = self._build_request_async(
            method="GET",
            path="/customers/wallets",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getCustomerWallets",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                List[models.components.DtoWalletResponse], http_res
            )
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def get_wallets_by_customer_id(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.components.DtoWalletResponse]:
        r"""Get wallets by customer ID

        Use when showing a customer's wallets (e.g. balance overview by currency or in a billing portal). Supports optional expand for balance breakdown.

        :param id: Customer ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.GetWalletsByCustomerIDRequest(
            id=id,
        )

        req = self._build_request(
            method="GET",
            path="/customers/{id}/wallets",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getWalletsByCustomerId",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                List[models.components.DtoWalletResponse], http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def get_wallets_by_customer_id_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.components.DtoWalletResponse]:
        r"""Get wallets by customer ID

        Use when showing a customer's wallets (e.g. balance overview by currency or in a billing portal). Supports optional expand for balance breakdown.

        :param id: Customer ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.GetWalletsByCustomerIDRequest(
            id=id,
        )

        req = self._build_request_async(
            method="GET",
            path="/customers/{id}/wallets",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getWalletsByCustomerId",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                List[models.components.DtoWalletResponse], http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def create_wallet(
        self,
        *,
        currency: str,
        alert_settings: Optional[
            Union[
                models.components.AlertSettings,
                models.components.AlertSettingsTypedDict,
            ]
        ] = None,
        auto_topup: Optional[
            Union[models.components.AutoTopup, models.components.AutoTopupTypedDict]
        ] = None,
        conversion_rate: Optional[str] = "1",
        customer_id: Optional[str] = None,
        description: Optional[str] = None,
        external_customer_id: Optional[str] = None,
        initial_credits_expiry_date_utc: Optional[str] = None,
        initial_credits_to_load: Optional[str] = "0",
        initial_credits_to_load_expiry_date: Optional[int] = None,
        metadata: Optional[Dict[str, str]] = None,
        price_unit: Optional[str] = None,
        topup_conversion_rate: Optional[str] = None,
        wallet_type: Optional[models.components.WalletType] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoWalletResponse:
        r"""Create a new wallet

        Use when giving a customer a prepaid or credit balance (e.g. prepaid plans or promotional credits).

        :param currency:
        :param alert_settings:
        :param auto_topup:
        :param conversion_rate: amount in the currency =  number of credits * conversion_rate
            ex if conversion_rate is 1, then 1 USD = 1 credit
            ex if conversion_rate is 2, then 1 USD = 0.5 credits
            ex if conversion_rate is 0.5, then 1 USD = 2 credits
        :param customer_id:
        :param description:
        :param external_customer_id: external_customer_id is the customer id in the external system
        :param initial_credits_expiry_date_utc: initial_credits_expiry_date_utc is the expiry date in UTC timezone (optional to set nil means no expiry)
            ex 2025-01-01 00:00:00 UTC
        :param initial_credits_to_load: initial_credits_to_load is the number of credits to load to the wallet
            if not provided, the wallet will be created with 0 balance
            NOTE: this is not the amount in the currency, but the number of credits
        :param initial_credits_to_load_expiry_date: initial_credits_to_load_expiry_date YYYYMMDD format in UTC timezone (optional to set nil means no expiry)
            for ex 20250101 means the credits will expire on 2025-01-01 00:00:00 UTC
            hence they will be available for use until 2024-12-31 23:59:59 UTC
        :param metadata:
        :param price_unit: price_unit is the code of the price unit to use for wallet creation
            If provided, the price unit will be used to set the currency and conversion rate of the wallet:
            - currency: set to price unit's base_currency
            - conversion_rate: set to price unit's conversion_rate
        :param topup_conversion_rate: topup_conversion_rate is the conversion rate for the topup to the currency
            ex if topup_conversion_rate is 1, then 1 USD = 1 credit
            ex if topup_conversion_rate is 2, then 1 USD = 0.5 credits
            ex if topup_conversion_rate is 0.5, then 1 USD = 2 credits
        :param wallet_type:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.DtoCreateWalletRequest(
            alert_settings=utils.get_pydantic_model(
                alert_settings, Optional[models.components.AlertSettings]
            ),
            auto_topup=utils.get_pydantic_model(
                auto_topup, Optional[models.components.AutoTopup]
            ),
            conversion_rate=conversion_rate,
            currency=currency,
            customer_id=customer_id,
            description=description,
            external_customer_id=external_customer_id,
            initial_credits_expiry_date_utc=initial_credits_expiry_date_utc,
            initial_credits_to_load=initial_credits_to_load,
            initial_credits_to_load_expiry_date=initial_credits_to_load_expiry_date,
            metadata=metadata,
            price_unit=price_unit,
            topup_conversion_rate=topup_conversion_rate,
            wallet_type=wallet_type,
        )

        req = self._build_request(
            method="POST",
            path="/wallets",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.components.DtoCreateWalletRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createWallet",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoWalletResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def create_wallet_async(
        self,
        *,
        currency: str,
        alert_settings: Optional[
            Union[
                models.components.AlertSettings,
                models.components.AlertSettingsTypedDict,
            ]
        ] = None,
        auto_topup: Optional[
            Union[models.components.AutoTopup, models.components.AutoTopupTypedDict]
        ] = None,
        conversion_rate: Optional[str] = "1",
        customer_id: Optional[str] = None,
        description: Optional[str] = None,
        external_customer_id: Optional[str] = None,
        initial_credits_expiry_date_utc: Optional[str] = None,
        initial_credits_to_load: Optional[str] = "0",
        initial_credits_to_load_expiry_date: Optional[int] = None,
        metadata: Optional[Dict[str, str]] = None,
        price_unit: Optional[str] = None,
        topup_conversion_rate: Optional[str] = None,
        wallet_type: Optional[models.components.WalletType] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoWalletResponse:
        r"""Create a new wallet

        Use when giving a customer a prepaid or credit balance (e.g. prepaid plans or promotional credits).

        :param currency:
        :param alert_settings:
        :param auto_topup:
        :param conversion_rate: amount in the currency =  number of credits * conversion_rate
            ex if conversion_rate is 1, then 1 USD = 1 credit
            ex if conversion_rate is 2, then 1 USD = 0.5 credits
            ex if conversion_rate is 0.5, then 1 USD = 2 credits
        :param customer_id:
        :param description:
        :param external_customer_id: external_customer_id is the customer id in the external system
        :param initial_credits_expiry_date_utc: initial_credits_expiry_date_utc is the expiry date in UTC timezone (optional to set nil means no expiry)
            ex 2025-01-01 00:00:00 UTC
        :param initial_credits_to_load: initial_credits_to_load is the number of credits to load to the wallet
            if not provided, the wallet will be created with 0 balance
            NOTE: this is not the amount in the currency, but the number of credits
        :param initial_credits_to_load_expiry_date: initial_credits_to_load_expiry_date YYYYMMDD format in UTC timezone (optional to set nil means no expiry)
            for ex 20250101 means the credits will expire on 2025-01-01 00:00:00 UTC
            hence they will be available for use until 2024-12-31 23:59:59 UTC
        :param metadata:
        :param price_unit: price_unit is the code of the price unit to use for wallet creation
            If provided, the price unit will be used to set the currency and conversion rate of the wallet:
            - currency: set to price unit's base_currency
            - conversion_rate: set to price unit's conversion_rate
        :param topup_conversion_rate: topup_conversion_rate is the conversion rate for the topup to the currency
            ex if topup_conversion_rate is 1, then 1 USD = 1 credit
            ex if topup_conversion_rate is 2, then 1 USD = 0.5 credits
            ex if topup_conversion_rate is 0.5, then 1 USD = 2 credits
        :param wallet_type:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.DtoCreateWalletRequest(
            alert_settings=utils.get_pydantic_model(
                alert_settings, Optional[models.components.AlertSettings]
            ),
            auto_topup=utils.get_pydantic_model(
                auto_topup, Optional[models.components.AutoTopup]
            ),
            conversion_rate=conversion_rate,
            currency=currency,
            customer_id=customer_id,
            description=description,
            external_customer_id=external_customer_id,
            initial_credits_expiry_date_utc=initial_credits_expiry_date_utc,
            initial_credits_to_load=initial_credits_to_load,
            initial_credits_to_load_expiry_date=initial_credits_to_load_expiry_date,
            metadata=metadata,
            price_unit=price_unit,
            topup_conversion_rate=topup_conversion_rate,
            wallet_type=wallet_type,
        )

        req = self._build_request_async(
            method="POST",
            path="/wallets",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.components.DtoCreateWalletRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createWallet",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoWalletResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def query_wallet(
        self,
        *,
        alert_enabled: Optional[bool] = None,
        expand: Optional[str] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        order: Optional[models.components.WalletFilterOrder] = None,
        sort: Optional[str] = None,
        status: Optional[models.components.WalletStatus] = None,
        wallet_ids: Optional[List[str]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.ListResponseDtoWalletResponse:
        r"""Query wallets

        Use when listing or searching wallets (e.g. admin view or reporting). Returns a paginated list; supports filtering by customer and status.

        :param alert_enabled:
        :param expand:
        :param limit:
        :param offset:
        :param order:
        :param sort:
        :param status:
        :param wallet_ids:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.WalletFilter(
            alert_enabled=alert_enabled,
            expand=expand,
            limit=limit,
            offset=offset,
            order=order,
            sort=sort,
            status=status,
            wallet_ids=wallet_ids,
        )

        req = self._build_request(
            method="POST",
            path="/wallets/search",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.components.WalletFilter
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="queryWallet",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.ListResponseDtoWalletResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def query_wallet_async(
        self,
        *,
        alert_enabled: Optional[bool] = None,
        expand: Optional[str] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        order: Optional[models.components.WalletFilterOrder] = None,
        sort: Optional[str] = None,
        status: Optional[models.components.WalletStatus] = None,
        wallet_ids: Optional[List[str]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.ListResponseDtoWalletResponse:
        r"""Query wallets

        Use when listing or searching wallets (e.g. admin view or reporting). Returns a paginated list; supports filtering by customer and status.

        :param alert_enabled:
        :param expand:
        :param limit:
        :param offset:
        :param order:
        :param sort:
        :param status:
        :param wallet_ids:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.WalletFilter(
            alert_enabled=alert_enabled,
            expand=expand,
            limit=limit,
            offset=offset,
            order=order,
            sort=sort,
            status=status,
            wallet_ids=wallet_ids,
        )

        req = self._build_request_async(
            method="POST",
            path="/wallets/search",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.components.WalletFilter
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="queryWallet",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.ListResponseDtoWalletResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def query_wallet_transaction(
        self,
        *,
        created_by: Optional[str] = None,
        credits_available_gt: Optional[float] = None,
        end_time: Optional[str] = None,
        expand: Optional[str] = None,
        expiry_date_after: Optional[str] = None,
        expiry_date_before: Optional[str] = None,
        filters: Optional[
            Union[
                List[models.components.FilterCondition],
                List[models.components.FilterConditionTypedDict],
            ]
        ] = None,
        id: Optional[str] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        order: Optional[models.components.WalletTransactionFilterOrder] = None,
        priority: Optional[int] = None,
        reference_id: Optional[str] = None,
        reference_type: Optional[str] = None,
        sort: Optional[
            Union[
                List[models.components.SortCondition],
                List[models.components.SortConditionTypedDict],
            ]
        ] = None,
        start_time: Optional[str] = None,
        status: Optional[models.components.Status] = None,
        transaction_reason: Optional[models.components.TransactionReason] = None,
        transaction_status: Optional[models.components.TransactionStatus] = None,
        type_: Optional[models.components.TransactionType] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoListWalletTransactionsResponse:
        r"""Query wallet transactions

        Use when searching or reporting on wallet transactions (e.g. cross-wallet history or reconciliation). Returns a paginated list; supports filtering by wallet, customer, type, date range.

        :param created_by:
        :param credits_available_gt:
        :param end_time:
        :param expand:
        :param expiry_date_after:
        :param expiry_date_before:
        :param filters: filters allows complex filtering based on multiple fields
        :param id:
        :param limit:
        :param offset:
        :param order:
        :param priority:
        :param reference_id:
        :param reference_type:
        :param sort:
        :param start_time:
        :param status:
        :param transaction_reason:
        :param transaction_status:
        :param type:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.WalletTransactionFilter(
            created_by=created_by,
            credits_available_gt=credits_available_gt,
            end_time=end_time,
            expand=expand,
            expiry_date_after=expiry_date_after,
            expiry_date_before=expiry_date_before,
            filters=utils.get_pydantic_model(
                filters, Optional[List[models.components.FilterCondition]]
            ),
            id=id,
            limit=limit,
            offset=offset,
            order=order,
            priority=priority,
            reference_id=reference_id,
            reference_type=reference_type,
            sort=utils.get_pydantic_model(
                sort, Optional[List[models.components.SortCondition]]
            ),
            start_time=start_time,
            status=status,
            transaction_reason=transaction_reason,
            transaction_status=transaction_status,
            type=type_,
        )

        req = self._build_request(
            method="POST",
            path="/wallets/transactions/search",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.components.WalletTransactionFilter
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="queryWalletTransaction",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoListWalletTransactionsResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def query_wallet_transaction_async(
        self,
        *,
        created_by: Optional[str] = None,
        credits_available_gt: Optional[float] = None,
        end_time: Optional[str] = None,
        expand: Optional[str] = None,
        expiry_date_after: Optional[str] = None,
        expiry_date_before: Optional[str] = None,
        filters: Optional[
            Union[
                List[models.components.FilterCondition],
                List[models.components.FilterConditionTypedDict],
            ]
        ] = None,
        id: Optional[str] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        order: Optional[models.components.WalletTransactionFilterOrder] = None,
        priority: Optional[int] = None,
        reference_id: Optional[str] = None,
        reference_type: Optional[str] = None,
        sort: Optional[
            Union[
                List[models.components.SortCondition],
                List[models.components.SortConditionTypedDict],
            ]
        ] = None,
        start_time: Optional[str] = None,
        status: Optional[models.components.Status] = None,
        transaction_reason: Optional[models.components.TransactionReason] = None,
        transaction_status: Optional[models.components.TransactionStatus] = None,
        type_: Optional[models.components.TransactionType] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoListWalletTransactionsResponse:
        r"""Query wallet transactions

        Use when searching or reporting on wallet transactions (e.g. cross-wallet history or reconciliation). Returns a paginated list; supports filtering by wallet, customer, type, date range.

        :param created_by:
        :param credits_available_gt:
        :param end_time:
        :param expand:
        :param expiry_date_after:
        :param expiry_date_before:
        :param filters: filters allows complex filtering based on multiple fields
        :param id:
        :param limit:
        :param offset:
        :param order:
        :param priority:
        :param reference_id:
        :param reference_type:
        :param sort:
        :param start_time:
        :param status:
        :param transaction_reason:
        :param transaction_status:
        :param type:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.WalletTransactionFilter(
            created_by=created_by,
            credits_available_gt=credits_available_gt,
            end_time=end_time,
            expand=expand,
            expiry_date_after=expiry_date_after,
            expiry_date_before=expiry_date_before,
            filters=utils.get_pydantic_model(
                filters, Optional[List[models.components.FilterCondition]]
            ),
            id=id,
            limit=limit,
            offset=offset,
            order=order,
            priority=priority,
            reference_id=reference_id,
            reference_type=reference_type,
            sort=utils.get_pydantic_model(
                sort, Optional[List[models.components.SortCondition]]
            ),
            start_time=start_time,
            status=status,
            transaction_reason=transaction_reason,
            transaction_status=transaction_status,
            type=type_,
        )

        req = self._build_request_async(
            method="POST",
            path="/wallets/transactions/search",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.components.WalletTransactionFilter
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="queryWalletTransaction",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoListWalletTransactionsResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def get_wallet(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoWalletResponse:
        r"""Get wallet

        Use when you need to load a single wallet (e.g. for a balance or settings view).

        :param id: Wallet ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.GetWalletRequest(
            id=id,
        )

        req = self._build_request(
            method="GET",
            path="/wallets/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getWallet",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoWalletResponse, http_res
            )
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def get_wallet_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoWalletResponse:
        r"""Get wallet

        Use when you need to load a single wallet (e.g. for a balance or settings view).

        :param id: Wallet ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.GetWalletRequest(
            id=id,
        )

        req = self._build_request_async(
            method="GET",
            path="/wallets/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getWallet",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoWalletResponse, http_res
            )
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def update_wallet(
        self,
        *,
        id: str,
        alert_settings: Optional[
            Union[
                models.components.AlertSettings,
                models.components.AlertSettingsTypedDict,
            ]
        ] = None,
        auto_topup: Optional[
            Union[models.components.AutoTopup, models.components.AutoTopupTypedDict]
        ] = None,
        config: Optional[
            Union[
                models.components.WalletConfig, models.components.WalletConfigTypedDict
            ]
        ] = None,
        description: Optional[str] = None,
        metadata: Optional[Dict[str, str]] = None,
        name: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoWalletResponse:
        r"""Update a wallet

        Use when changing wallet settings (e.g. enabling or updating auto top-up thresholds).

        :param id: Wallet ID
        :param alert_settings:
        :param auto_topup:
        :param config:
        :param description:
        :param metadata:
        :param name:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.UpdateWalletRequest(
            id=id,
            body=models.components.DtoUpdateWalletRequest(
                alert_settings=utils.get_pydantic_model(
                    alert_settings, Optional[models.components.AlertSettings]
                ),
                auto_topup=utils.get_pydantic_model(
                    auto_topup, Optional[models.components.AutoTopup]
                ),
                config=utils.get_pydantic_model(
                    config, Optional[models.components.WalletConfig]
                ),
                description=description,
                metadata=metadata,
                name=name,
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/wallets/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body,
                False,
                False,
                "json",
                models.components.DtoUpdateWalletRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updateWallet",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoWalletResponse, http_res
            )
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def update_wallet_async(
        self,
        *,
        id: str,
        alert_settings: Optional[
            Union[
                models.components.AlertSettings,
                models.components.AlertSettingsTypedDict,
            ]
        ] = None,
        auto_topup: Optional[
            Union[models.components.AutoTopup, models.components.AutoTopupTypedDict]
        ] = None,
        config: Optional[
            Union[
                models.components.WalletConfig, models.components.WalletConfigTypedDict
            ]
        ] = None,
        description: Optional[str] = None,
        metadata: Optional[Dict[str, str]] = None,
        name: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoWalletResponse:
        r"""Update a wallet

        Use when changing wallet settings (e.g. enabling or updating auto top-up thresholds).

        :param id: Wallet ID
        :param alert_settings:
        :param auto_topup:
        :param config:
        :param description:
        :param metadata:
        :param name:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.UpdateWalletRequest(
            id=id,
            body=models.components.DtoUpdateWalletRequest(
                alert_settings=utils.get_pydantic_model(
                    alert_settings, Optional[models.components.AlertSettings]
                ),
                auto_topup=utils.get_pydantic_model(
                    auto_topup, Optional[models.components.AutoTopup]
                ),
                config=utils.get_pydantic_model(
                    config, Optional[models.components.WalletConfig]
                ),
                description=description,
                metadata=metadata,
                name=name,
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/wallets/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body,
                False,
                False,
                "json",
                models.components.DtoUpdateWalletRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updateWallet",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoWalletResponse, http_res
            )
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def get_wallet_balance(
        self,
        *,
        id: str,
        expand: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoWalletBalanceResponse:
        r"""Get wallet balance

        Use when displaying or checking current wallet balance (e.g. before charging or in a portal). Supports optional expand for credits breakdown and from_cache.

        :param id: Wallet ID
        :param expand: Expand fields (e.g., credits_available_breakdown)
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.GetWalletBalanceRequest(
            id=id,
            expand=expand,
        )

        req = self._build_request(
            method="GET",
            path="/wallets/{id}/balance/real-time",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getWalletBalance",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoWalletBalanceResponse, http_res
            )
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def get_wallet_balance_async(
        self,
        *,
        id: str,
        expand: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoWalletBalanceResponse:
        r"""Get wallet balance

        Use when displaying or checking current wallet balance (e.g. before charging or in a portal). Supports optional expand for credits breakdown and from_cache.

        :param id: Wallet ID
        :param expand: Expand fields (e.g., credits_available_breakdown)
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.GetWalletBalanceRequest(
            id=id,
            expand=expand,
        )

        req = self._build_request_async(
            method="GET",
            path="/wallets/{id}/balance/real-time",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getWalletBalance",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoWalletBalanceResponse, http_res
            )
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def terminate_wallet(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoWalletResponse:
        r"""Terminate a wallet

        Use when closing a customer wallet (e.g. churn or migration). Closes the wallet and applies remaining balance per policy (refund or forfeit).

        :param id: Wallet ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.TerminateWalletRequest(
            id=id,
        )

        req = self._build_request(
            method="POST",
            path="/wallets/{id}/terminate",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="terminateWallet",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoWalletResponse, http_res
            )
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def terminate_wallet_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoWalletResponse:
        r"""Terminate a wallet

        Use when closing a customer wallet (e.g. churn or migration). Closes the wallet and applies remaining balance per policy (refund or forfeit).

        :param id: Wallet ID
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.TerminateWalletRequest(
            id=id,
        )

        req = self._build_request_async(
            method="POST",
            path="/wallets/{id}/terminate",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="terminateWallet",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoWalletResponse, http_res
            )
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def top_up_wallet(
        self,
        *,
        id: str,
        transaction_reason: models.components.TransactionReason,
        amount: Optional[str] = None,
        credits_to_add: Optional[str] = None,
        description: Optional[str] = None,
        expiry_date_utc: Optional[str] = None,
        idempotency_key: Optional[str] = None,
        metadata: Optional[Dict[str, str]] = None,
        priority: Optional[int] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoTopUpWalletResponse:
        r"""Top up wallet

        Use when adding funds to a wallet (e.g. top-up, refund, or manual credit). Supports optional idempotency via reference.

        :param id: Wallet ID
        :param transaction_reason:
        :param amount: amount is the amount in the currency of the wallet to be added
            NOTE: this is not the number of credits to add, but the amount in the currency
            amount = credits_to_add * conversion_rate
            if both amount and credits_to_add are provided, amount will be ignored
            ex if the wallet has a conversion_rate of 2 then adding an amount of
            10 USD in the wallet wil add 5 credits in the wallet
        :param credits_to_add: credits_to_add is the number of credits to add to the wallet
        :param description: description to add any specific details about the transaction
        :param expiry_date_utc: expiry_date_utc is the expiry date in UTC timezone
            ex 2025-01-01 00:00:00 UTC
        :param idempotency_key: idempotency_key is a unique key for the transaction
        :param metadata:
        :param priority: priority is the priority of the transaction
            lower number means higher priority
            default is nil which means no priority at all
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.TopUpWalletRequest(
            id=id,
            body=models.components.DtoTopUpWalletRequest(
                amount=amount,
                credits_to_add=credits_to_add,
                description=description,
                expiry_date_utc=expiry_date_utc,
                idempotency_key=idempotency_key,
                metadata=metadata,
                priority=priority,
                transaction_reason=transaction_reason,
            ),
        )

        req = self._build_request(
            method="POST",
            path="/wallets/{id}/top-up",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body,
                False,
                False,
                "json",
                models.components.DtoTopUpWalletRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="topUpWallet",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoTopUpWalletResponse, http_res
            )
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def top_up_wallet_async(
        self,
        *,
        id: str,
        transaction_reason: models.components.TransactionReason,
        amount: Optional[str] = None,
        credits_to_add: Optional[str] = None,
        description: Optional[str] = None,
        expiry_date_utc: Optional[str] = None,
        idempotency_key: Optional[str] = None,
        metadata: Optional[Dict[str, str]] = None,
        priority: Optional[int] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoTopUpWalletResponse:
        r"""Top up wallet

        Use when adding funds to a wallet (e.g. top-up, refund, or manual credit). Supports optional idempotency via reference.

        :param id: Wallet ID
        :param transaction_reason:
        :param amount: amount is the amount in the currency of the wallet to be added
            NOTE: this is not the number of credits to add, but the amount in the currency
            amount = credits_to_add * conversion_rate
            if both amount and credits_to_add are provided, amount will be ignored
            ex if the wallet has a conversion_rate of 2 then adding an amount of
            10 USD in the wallet wil add 5 credits in the wallet
        :param credits_to_add: credits_to_add is the number of credits to add to the wallet
        :param description: description to add any specific details about the transaction
        :param expiry_date_utc: expiry_date_utc is the expiry date in UTC timezone
            ex 2025-01-01 00:00:00 UTC
        :param idempotency_key: idempotency_key is a unique key for the transaction
        :param metadata:
        :param priority: priority is the priority of the transaction
            lower number means higher priority
            default is nil which means no priority at all
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.TopUpWalletRequest(
            id=id,
            body=models.components.DtoTopUpWalletRequest(
                amount=amount,
                credits_to_add=credits_to_add,
                description=description,
                expiry_date_utc=expiry_date_utc,
                idempotency_key=idempotency_key,
                metadata=metadata,
                priority=priority,
                transaction_reason=transaction_reason,
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/wallets/{id}/top-up",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body,
                False,
                False,
                "json",
                models.components.DtoTopUpWalletRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="topUpWallet",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoTopUpWalletResponse, http_res
            )
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    def get_wallet_transactions(
        self,
        *,
        id_path_parameter: str,
        created_by: Optional[str] = None,
        credits_available_gt: Optional[float] = None,
        end_time: Optional[str] = None,
        expand: Optional[str] = None,
        expiry_date_after: Optional[str] = None,
        expiry_date_before: Optional[str] = None,
        id_query_parameter: Optional[str] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        order: Optional[models.operations.GetWalletTransactionsOrder] = None,
        priority: Optional[int] = None,
        reference_id: Optional[str] = None,
        reference_type: Optional[str] = None,
        start_time: Optional[str] = None,
        status: Optional[models.operations.GetWalletTransactionsStatus] = None,
        transaction_reason: Optional[models.operations.TransactionReason] = None,
        transaction_status: Optional[models.operations.TransactionStatus] = None,
        type_: Optional[models.operations.Type] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoListWalletTransactionsResponse:
        r"""Get wallet transactions

        Use when showing transaction history for a wallet (e.g. credit/debit log or audit). Returns a paginated list; supports limit, offset, and filters.

        :param id_path_parameter: Wallet ID
        :param created_by:
        :param credits_available_gt:
        :param end_time:
        :param expand:
        :param expiry_date_after:
        :param expiry_date_before:
        :param id_query_parameter:
        :param limit:
        :param offset:
        :param order:
        :param priority:
        :param reference_id:
        :param reference_type:
        :param start_time:
        :param status:
        :param transaction_reason:
        :param transaction_status:
        :param type:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.GetWalletTransactionsRequest(
            id_path_parameter=id_path_parameter,
            created_by=created_by,
            credits_available_gt=credits_available_gt,
            end_time=end_time,
            expand=expand,
            expiry_date_after=expiry_date_after,
            expiry_date_before=expiry_date_before,
            id_query_parameter=id_query_parameter,
            limit=limit,
            offset=offset,
            order=order,
            priority=priority,
            reference_id=reference_id,
            reference_type=reference_type,
            start_time=start_time,
            status=status,
            transaction_reason=transaction_reason,
            transaction_status=transaction_status,
            type=type_,
        )

        req = self._build_request(
            method="GET",
            path="/wallets/{id}/transactions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getWalletTransactions",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoListWalletTransactionsResponse, http_res
            )
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def get_wallet_transactions_async(
        self,
        *,
        id_path_parameter: str,
        created_by: Optional[str] = None,
        credits_available_gt: Optional[float] = None,
        end_time: Optional[str] = None,
        expand: Optional[str] = None,
        expiry_date_after: Optional[str] = None,
        expiry_date_before: Optional[str] = None,
        id_query_parameter: Optional[str] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        order: Optional[models.operations.GetWalletTransactionsOrder] = None,
        priority: Optional[int] = None,
        reference_id: Optional[str] = None,
        reference_type: Optional[str] = None,
        start_time: Optional[str] = None,
        status: Optional[models.operations.GetWalletTransactionsStatus] = None,
        transaction_reason: Optional[models.operations.TransactionReason] = None,
        transaction_status: Optional[models.operations.TransactionStatus] = None,
        type_: Optional[models.operations.Type] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoListWalletTransactionsResponse:
        r"""Get wallet transactions

        Use when showing transaction history for a wallet (e.g. credit/debit log or audit). Returns a paginated list; supports limit, offset, and filters.

        :param id_path_parameter: Wallet ID
        :param created_by:
        :param credits_available_gt:
        :param end_time:
        :param expand:
        :param expiry_date_after:
        :param expiry_date_before:
        :param id_query_parameter:
        :param limit:
        :param offset:
        :param order:
        :param priority:
        :param reference_id:
        :param reference_type:
        :param start_time:
        :param status:
        :param transaction_reason:
        :param transaction_status:
        :param type:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.operations.GetWalletTransactionsRequest(
            id_path_parameter=id_path_parameter,
            created_by=created_by,
            credits_available_gt=credits_available_gt,
            end_time=end_time,
            expand=expand,
            expiry_date_after=expiry_date_after,
            expiry_date_before=expiry_date_before,
            id_query_parameter=id_query_parameter,
            limit=limit,
            offset=offset,
            order=order,
            priority=priority,
            reference_id=reference_id,
            reference_type=reference_type,
            start_time=start_time,
            status=status,
            transaction_reason=transaction_reason,
            transaction_status=transaction_status,
            type=type_,
        )

        req = self._build_request_async(
            method="GET",
            path="/wallets/{id}/transactions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getWalletTransactions",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoListWalletTransactionsResponse, http_res
            )
        if utils.match_response(http_res, ["400", "404"], "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .integrationentitytype import IntegrationEntityType
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Any, Dict, Optional
from typing_extensions import NotRequired, TypedDict


class DtoCreateEntityIntegrationMappingRequestTypedDict(TypedDict):
    entity_id: str
    entity_type: IntegrationEntityType
    provider_entity_id: str
    provider_type: str
    metadata: NotRequired[Dict[str, Any]]


class DtoCreateEntityIntegrationMappingRequest(BaseModel):
    entity_id: str

    entity_type: IntegrationEntityType

    provider_entity_id: str

    provider_type: str

    metadata: Optional[Dict[str, Any]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["metadata"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from flexprice_py import errors, models, utils
from flexprice_py._hooks import HookContext
from flexprice_py.types import OptionalNullable, UNSET
from flexprice_py.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, List, Mapping, Optional, Union


class Workflows(BaseSDK):
    def query_workflow(
        self,
        *,
        end_time: Optional[str] = None,
        entity: Optional[str] = None,
        entity_id: Optional[str] = None,
        expand: Optional[str] = None,
        filters: Optional[
            Union[
                List[models.components.FilterCondition],
                List[models.components.FilterConditionTypedDict],
            ]
        ] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        order: Optional[models.components.WorkflowExecutionFilterOrder] = None,
        sort: Optional[
            Union[
                List[models.components.SortCondition],
                List[models.components.SortConditionTypedDict],
            ]
        ] = None,
        start_time: Optional[str] = None,
        status: Optional[models.components.Status] = None,
        task_queue: Optional[str] = None,
        workflow_id: Optional[str] = None,
        workflow_status: Optional[str] = None,
        workflow_type: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoListWorkflowsResponse:
        r"""Query workflows

        Use when listing or auditing workflow runs (e.g. ops dashboard or debugging). Returns a paginated list; supports filtering by workflow type and status.

        :param end_time:
        :param entity: e.g. plan, invoice, subscription
        :param entity_id: e.g. plan_01ABC123
        :param expand:
        :param filters: filters allows complex filtering based on multiple fields (same as FeatureFilter)
        :param limit:
        :param offset:
        :param order:
        :param sort:
        :param start_time:
        :param status:
        :param task_queue:
        :param workflow_id: Workflow-specific filters
        :param workflow_status: e.g. Running, Completed, Failed
        :param workflow_type:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.WorkflowExecutionFilter(
            end_time=end_time,
            entity=entity,
            entity_id=entity_id,
            expand=expand,
            filters=utils.get_pydantic_model(
                filters, Optional[List[models.components.FilterCondition]]
            ),
            limit=limit,
            offset=offset,
            order=order,
            sort=utils.get_pydantic_model(
                sort, Optional[List[models.components.SortCondition]]
            ),
            start_time=start_time,
            status=status,
            task_queue=task_queue,
            workflow_id=workflow_id,
            workflow_status=workflow_status,
            workflow_type=workflow_type,
        )

        req = self._build_request(
            method="POST",
            path="/workflows/search",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.components.WorkflowExecutionFilter
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="queryWorkflow",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoListWorkflowsResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)

    async def query_workflow_async(
        self,
        *,
        end_time: Optional[str] = None,
        entity: Optional[str] = None,
        entity_id: Optional[str] = None,
        expand: Optional[str] = None,
        filters: Optional[
            Union[
                List[models.components.FilterCondition],
                List[models.components.FilterConditionTypedDict],
            ]
        ] = None,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
        order: Optional[models.components.WorkflowExecutionFilterOrder] = None,
        sort: Optional[
            Union[
                List[models.components.SortCondition],
                List[models.components.SortConditionTypedDict],
            ]
        ] = None,
        start_time: Optional[str] = None,
        status: Optional[models.components.Status] = None,
        task_queue: Optional[str] = None,
        workflow_id: Optional[str] = None,
        workflow_status: Optional[str] = None,
        workflow_type: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.components.DtoListWorkflowsResponse:
        r"""Query workflows

        Use when listing or auditing workflow runs (e.g. ops dashboard or debugging). Returns a paginated list; supports filtering by workflow type and status.

        :param end_time:
        :param entity: e.g. plan, invoice, subscription
        :param entity_id: e.g. plan_01ABC123
        :param expand:
        :param filters: filters allows complex filtering based on multiple fields (same as FeatureFilter)
        :param limit:
        :param offset:
        :param order:
        :param sort:
        :param start_time:
        :param status:
        :param task_queue:
        :param workflow_id: Workflow-specific filters
        :param workflow_status: e.g. Running, Completed, Failed
        :param workflow_type:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.components.WorkflowExecutionFilter(
            end_time=end_time,
            entity=entity,
            entity_id=entity_id,
            expand=expand,
            filters=utils.get_pydantic_model(
                filters, Optional[List[models.components.FilterCondition]]
            ),
            limit=limit,
            offset=offset,
            order=order,
            sort=utils.get_pydantic_model(
                sort, Optional[List[models.components.SortCondition]]
            ),
            start_time=start_time,
            status=status,
            task_queue=task_queue,
            workflow_id=workflow_id,
            workflow_status=workflow_status,
            workflow_type=workflow_type,
        )

        req = self._build_request_async(
            method="POST",
            path="/workflows/search",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.components.WorkflowExecutionFilter
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="queryWorkflow",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.components.DtoListWorkflowsResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.ErrorsErrorResponseData, http_res
            )
            raise errors.ErrorsErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.FlexpriceDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.FlexpriceDefaultError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .filtercondition import FilterCondition, FilterConditionTypedDict
from .queryfilter import QueryFilter, QueryFilterTypedDict
from .sortcondition import SortCondition, SortConditionTypedDict
from .status import Status
from .timerangefilter import TimeRangeFilter, TimeRangeFilterTypedDict
from flexprice_py.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CostsheetFilterTypedDict(TypedDict):
    costsheet_i_ds: NotRequired[List[str]]
    r"""CostsheetIDs allows filtering by specific costsheet IDs"""
    environment_id: NotRequired[str]
    r"""EnvironmentID filters by specific environment ID"""
    filters: NotRequired[List[FilterConditionTypedDict]]
    r"""Filters contains custom filtering conditions"""
    lookup_key: NotRequired[str]
    r"""LookupKey filters by lookup key"""
    name: NotRequired[str]
    r"""Name filters by costsheet name"""
    query_filter: NotRequired[QueryFilterTypedDict]
    sort: NotRequired[List[SortConditionTypedDict]]
    r"""Sort specifies result ordering preferences"""
    status: NotRequired[Status]
    tenant_id: NotRequired[str]
    r"""TenantID filters by specific tenant ID"""
    time_range_filter: NotRequired[TimeRangeFilterTypedDict]


class CostsheetFilter(BaseModel):
    costsheet_i_ds: Annotated[
        Optional[List[str]], pydantic.Field(alias="costsheetIDs")
    ] = None
    r"""CostsheetIDs allows filtering by specific costsheet IDs"""

    environment_id: Annotated[Optional[str], pydantic.Field(alias="environmentID")] = (
        None
    )
    r"""EnvironmentID filters by specific environment ID"""

    filters: Optional[List[FilterCondition]] = None
    r"""Filters contains custom filtering conditions"""

    lookup_key: Annotated[Optional[str], pydantic.Field(alias="lookupKey")] = None
    r"""LookupKey filters by lookup key"""

    name: Optional[str] = None
    r"""Name filters by costsheet name"""

    query_filter: Annotated[
        Optional[QueryFilter], pydantic.Field(alias="queryFilter")
    ] = None

    sort: Optional[List[SortCondition]] = None
    r"""Sort specifies result ordering preferences"""

    status: Optional[Status] = None

    tenant_id: Annotated[Optional[str], pydantic.Field(alias="tenantID")] = None
    r"""TenantID filters by specific tenant ID"""

    time_range_filter: Annotated[
        Optional[TimeRangeFilter], pydantic.Field(alias="timeRangeFilter")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "costsheetIDs",
                "environmentID",
                "filters",
                "lookupKey",
                "name",
                "queryFilter",
                "sort",
                "status",
                "tenantID",
                "timeRangeFilter",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    CostsheetFilter.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .resumemode import ResumeMode
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class DtoResumeSubscriptionRequestTypedDict(TypedDict):
    r"""Request object for resuming a paused subscription"""

    resume_mode: ResumeMode
    dry_run: NotRequired[bool]
    r"""Whether to perform a dry run
    @Description If true, validates the request and shows impact without actually resuming the subscription
    @Example false
    """
    metadata: NotRequired[Dict[str, str]]
    r"""Additional metadata as key-value pairs
    @Description Optional metadata for storing additional information about the resume operation
    @Example {\"resumed_by\": \"admin\", \"reason\": \"issue_resolved\"}
    """


class DtoResumeSubscriptionRequest(BaseModel):
    r"""Request object for resuming a paused subscription"""

    resume_mode: ResumeMode

    dry_run: Optional[bool] = None
    r"""Whether to perform a dry run
    @Description If true, validates the request and shows impact without actually resuming the subscription
    @Example false
    """

    metadata: Optional[Dict[str, str]] = None
    r"""Additional metadata as key-value pairs
    @Description Optional metadata for storing additional information about the resume operation
    @Example {\"resumed_by\": \"admin\", \"reason\": \"issue_resolved\"}
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["dry_run", "metadata"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .flexpriceerror import FlexpriceError
from typing import Any, TYPE_CHECKING

from flexprice_py.utils.dynamic_imports import lazy_getattr, lazy_dir

if TYPE_CHECKING:
    from .errors_errorresponse import ErrorsErrorResponse, ErrorsErrorResponseData
    from .flexpricedefaulterror import FlexpriceDefaultError
    from .no_response_error import NoResponseError
    from .responsevalidationerror import ResponseValidationError

__all__ = [
    "ErrorsErrorResponse",
    "ErrorsErrorResponseData",
    "FlexpriceDefaultError",
    "FlexpriceError",
    "NoResponseError",
    "ResponseValidationError",
]

_dynamic_imports: dict[str, str] = {
    "ErrorsErrorResponse": ".errors_errorresponse",
    "ErrorsErrorResponseData": ".errors_errorresponse",
    "FlexpriceDefaultError": ".flexpricedefaulterror",
    "NoResponseError": ".no_response_error",
    "ResponseValidationError": ".responsevalidationerror",
}


def __getattr__(attr_name: str) -> Any:
    return lazy_getattr(
        attr_name, package=__package__, dynamic_imports=_dynamic_imports
    )


def __dir__():
    return lazy_dir(dynamic_imports=_dynamic_imports)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dto_customanalyticitem import (
    DtoCustomAnalyticItem,
    DtoCustomAnalyticItemTypedDict,
)
from .dto_usageanalyticitem import DtoUsageAnalyticItem, DtoUsageAnalyticItemTypedDict
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class DtoGetUsageAnalyticsResponseTypedDict(TypedDict):
    currency: NotRequired[str]
    custom_analytics: NotRequired[List[DtoCustomAnalyticItemTypedDict]]
    items: NotRequired[List[DtoUsageAnalyticItemTypedDict]]
    total_cost: NotRequired[str]


class DtoGetUsageAnalyticsResponse(BaseModel):
    currency: Optional[str] = None

    custom_analytics: Optional[List[DtoCustomAnalyticItem]] = None

    items: Optional[List[DtoUsageAnalyticItem]] = None

    total_cost: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["currency", "custom_analytics", "items", "total_cost"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from flexprice_py.models.components import (
    dto_updatecustomerrequest as components_dto_updatecustomerrequest,
)
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from flexprice_py.utils import FieldMetadata, QueryParamMetadata, RequestMetadata
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class UpdateCustomerRequestTypedDict(TypedDict):
    body: components_dto_updatecustomerrequest.DtoUpdateCustomerRequestTypedDict
    r"""Customer"""
    id: NotRequired[str]
    r"""Customer ID"""
    external_customer_id: NotRequired[str]
    r"""Customer External ID"""


class UpdateCustomerRequest(BaseModel):
    body: Annotated[
        components_dto_updatecustomerrequest.DtoUpdateCustomerRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]
    r"""Customer"""

    id: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Customer ID"""

    external_customer_id: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Customer External ID"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["id", "external_customer_id"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .billingcadence import BillingCadence
from .billingcycle import BillingCycle
from .billingperiod import BillingPeriod
from .collectionmethod import CollectionMethod
from .dto_addaddontosubscriptionrequest import (
    DtoAddAddonToSubscriptionRequest,
    DtoAddAddonToSubscriptionRequestTypedDict,
)
from .dto_createcreditgrantrequest import (
    DtoCreateCreditGrantRequest,
    DtoCreateCreditGrantRequestTypedDict,
)
from .dto_createsubscriptionlineitemrequest import (
    DtoCreateSubscriptionLineItemRequest,
    DtoCreateSubscriptionLineItemRequestTypedDict,
)
from .dto_lineitemcommitmentconfig import (
    DtoLineItemCommitmentConfig,
    DtoLineItemCommitmentConfigTypedDict,
)
from .dto_overrideentitlementrequest import (
    DtoOverrideEntitlementRequest,
    DtoOverrideEntitlementRequestTypedDict,
)
from .dto_overridelineitemrequest import (
    DtoOverrideLineItemRequest,
    DtoOverrideLineItemRequestTypedDict,
)
from .dto_subscriptionphasecreaterequest import (
    DtoSubscriptionPhaseCreateRequest,
    DtoSubscriptionPhaseCreateRequestTypedDict,
)
from .dto_taxrateoverride import DtoTaxRateOverride, DtoTaxRateOverrideTypedDict
from .invoicebilling import InvoiceBilling
from .paymentbehavior import PaymentBehavior
from .paymentterms import PaymentTerms
from .prorationbehavior import ProrationBehavior
from .subscriptionstatus import SubscriptionStatus
from flexprice_py.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class DtoCreateSubscriptionRequestTypedDict(TypedDict):
    billing_cadence: BillingCadence
    billing_period: BillingPeriod
    currency: str
    plan_id: str
    addons: NotRequired[List[DtoAddAddonToSubscriptionRequestTypedDict]]
    r"""Addons represents addons to be added to the subscription during creation"""
    billing_cycle: NotRequired[BillingCycle]
    billing_period_count: NotRequired[int]
    collection_method: NotRequired[CollectionMethod]
    commitment_amount: NotRequired[str]
    r"""CommitmentAmount is the minimum amount a customer commits to paying for a billing period"""
    commitment_duration: NotRequired[BillingPeriod]
    coupons: NotRequired[List[str]]
    credit_grants: NotRequired[List[DtoCreateCreditGrantRequestTypedDict]]
    r"""Credit grants to be applied when subscription is created"""
    customer_id: NotRequired[str]
    r"""customer_id is the flexprice customer id
    and it is prioritized over external_customer_id in case both are provided.
    """
    customer_timezone: NotRequired[str]
    r"""Timezone of the customer.
    If not set, the default value is UTC.
    """
    enable_true_up: NotRequired[bool]
    r"""Enable Commitment True Up Fee"""
    end_date: NotRequired[str]
    external_customer_id: NotRequired[str]
    r"""external_customer_id is the customer id in your DB
    and must be same as what you provided as external_id while creating the customer in flexprice.
    """
    gateway_payment_method_id: NotRequired[str]
    invoice_billing: NotRequired[InvoiceBilling]
    line_item_commitments: NotRequired[Dict[str, DtoLineItemCommitmentConfigTypedDict]]
    r"""LineItemCommitments allows setting commitment configuration per line item (keyed by price_id)"""
    line_item_coupons: NotRequired[Dict[str, List[str]]]
    line_items: NotRequired[List[DtoCreateSubscriptionLineItemRequestTypedDict]]
    r"""LineItems are extra line items to add at creation (each with price_id or price), in addition to plan prices"""
    lookup_key: NotRequired[str]
    metadata: NotRequired[Dict[str, str]]
    overage_factor: NotRequired[str]
    r"""OverageFactor is a multiplier applied to usage beyond the commitment amount"""
    override_entitlements: NotRequired[List[DtoOverrideEntitlementRequestTypedDict]]
    r"""OverrideEntitlements allows customizing specific entitlements for this subscription"""
    override_line_items: NotRequired[List[DtoOverrideLineItemRequestTypedDict]]
    r"""OverrideLineItems allows customizing specific prices for this subscription"""
    parent_subscription_id: NotRequired[str]
    r"""ParentSubscriptionID is the parent subscription ID for hierarchy (e.g. child subscription under a parent)"""
    payment_behavior: NotRequired[PaymentBehavior]
    payment_terms: NotRequired[PaymentTerms]
    phases: NotRequired[List[DtoSubscriptionPhaseCreateRequestTypedDict]]
    r"""Phases represents subscription phases to be created with the subscription"""
    proration_behavior: NotRequired[ProrationBehavior]
    start_date: NotRequired[str]
    subscription_status: NotRequired[SubscriptionStatus]
    tax_rate_overrides: NotRequired[List[DtoTaxRateOverrideTypedDict]]
    r"""tax_rate_overrides is the tax rate overrides	to be applied to the subscription"""
    trial_end: NotRequired[str]
    trial_start: NotRequired[str]


class DtoCreateSubscriptionRequest(BaseModel):
    billing_cadence: BillingCadence

    billing_period: BillingPeriod

    currency: str

    plan_id: str

    addons: Optional[List[DtoAddAddonToSubscriptionRequest]] = None
    r"""Addons represents addons to be added to the subscription during creation"""

    billing_cycle: Optional[BillingCycle] = None

    billing_period_count: Optional[int] = None

    collection_method: Optional[CollectionMethod] = None

    commitment_amount: Optional[str] = None
    r"""CommitmentAmount is the minimum amount a customer commits to paying for a billing period"""

    commitment_duration: Optional[BillingPeriod] = None

    coupons: Optional[List[str]] = None

    credit_grants: Optional[List[DtoCreateCreditGrantRequest]] = None
    r"""Credit grants to be applied when subscription is created"""

    customer_id: Optional[str] = None
    r"""customer_id is the flexprice customer id
    and it is prioritized over external_customer_id in case both are provided.
    """

    customer_timezone: Optional[str] = None
    r"""Timezone of the customer.
    If not set, the default value is UTC.
    """

    enable_true_up: Optional[bool] = None
    r"""Enable Commitment True Up Fee"""

    end_date: Optional[str] = None

    external_customer_id: Optional[str] = None
    r"""external_customer_id is the customer id in your DB
    and must be same as what you provided as external_id while creating the customer in flexprice.
    """

    gateway_payment_method_id: Optional[str] = None

    invoice_billing: Optional[InvoiceBilling] = None

    line_item_commitments: Optional[Dict[str, DtoLineItemCommitmentConfig]] = None
    r"""LineItemCommitments allows setting commitment configuration per line item (keyed by price_id)"""

    line_item_coupons: Optional[Dict[str, List[str]]] = None

    line_items: Optional[List[DtoCreateSubscriptionLineItemRequest]] = None
    r"""LineItems are extra line items to add at creation (each with price_id or price), in addition to plan prices"""

    lookup_key: Optional[str] = None

    metadata: Optional[Dict[str, str]] = None

    overage_factor: Optional[str] = None
    r"""OverageFactor is a multiplier applied to usage beyond the commitment amount"""

    override_entitlements: Optional[List[DtoOverrideEntitlementRequest]] = None
    r"""OverrideEntitlements allows customizing specific entitlements for this subscription"""

    override_line_items: Optional[List[DtoOverrideLineItemRequest]] = None
    r"""OverrideLineItems allows customizing specific prices for this subscription"""

    parent_subscription_id: Optional[str] = None
    r"""ParentSubscriptionID is the parent subscription ID for hierarchy (e.g. child subscription under a parent)"""

    payment_behavior: Optional[PaymentBehavior] = None

    payment_terms: Optional[PaymentTerms] = None

    phases: Optional[List[DtoSubscriptionPhaseCreateRequest]] = None
    r"""Phases represents subscription phases to be created with the subscription"""

    proration_behavior: Optional[ProrationBehavior] = None

    start_date: Optional[str] = None

    subscription_status: Optional[SubscriptionStatus] = None

    tax_rate_overrides: Optional[List[DtoTaxRateOverride]] = None
    r"""tax_rate_overrides is the tax rate overrides	to be applied to the subscription"""

    trial_end: Optional[str] = None

    trial_start: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "addons",
                "billing_cycle",
                "billing_period_count",
                "collection_method",
                "commitment_amount",
                "commitment_duration",
                "coupons",
                "credit_grants",
                "customer_id",
                "customer_timezone",
                "enable_true_up",
                "end_date",
                "external_customer_id",
                "gateway_payment_method_id",
                "invoice_billing",
                "line_item_commitments",
                "line_item_coupons",
                "line_items",
                "lookup_key",
                "metadata",
                "overage_factor",
                "override_entitlements",
                "override_line_items",
                "parent_subscription_id",
                "payment_behavior",
                "payment_terms",
                "phases",
                "proration_behavior",
                "start_date",
                "subscription_status",
                "tax_rate_overrides",
                "trial_end",
                "trial_start",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m
